# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2023, by Samuel Williams.

require_relative 'time/quantum'
require_relative 'time/version'

# @namespace
module Sus
	# @namespace
	module Fixtures
		# @namespace
		module Time
		end
	end
end
